#include <string>
#include "BinarySearchTree.h"

using namespace std;

int BinarySearchTree::height(const BinaryNode *ptr) const
{
    return ptr == nullptr ? -1
                          : 1 + max(height(ptr->left),
                                    height(ptr->right));
}

BinaryNode *BinarySearchTree::find_min(BinaryNode *ptr) const
{
    if (ptr != nullptr)
    {
        while (ptr->left != nullptr) ptr = ptr->left;
    }

    return ptr;
}

BinaryNode *BinarySearchTree::find_max(BinaryNode *ptr) const
{
    if (ptr != nullptr)
    {
        while (ptr->right != nullptr) ptr = ptr->right;
    }

    return ptr;
}

//bool BinarySearchTree::contains(const int data, BinaryNode *ptr) const
//{
//    while (ptr != nullptr)
//    {
//        if      (data < ptr->data) ptr = ptr->left;
//        else if (data > ptr->data) ptr = ptr->right;
//        else return true;  // found
//    }
//
//    return false;         // not found
//}

bool BinarySearchTree::contains(const int data, BinaryNode *ptr) const
{
    if      (ptr == nullptr)    return false;
    else if (data == ptr->data) return true;
    else if (data < ptr->data)  return contains(data, ptr->left);
    else                        return contains(data, ptr->right);
}

void BinarySearchTree::insert(const int data, BinaryNode* &ptr)
{
    if      (ptr == nullptr)   ptr = new BinaryNode(data);
    else if (data < ptr->data) insert(data, ptr->left);
    else if (data > ptr->data) insert(data, ptr->right);
//  else                       update the node
}

void BinarySearchTree::remove(const int data, BinaryNode* &ptr)
{
    if (ptr == nullptr) return;   // not found, so do nothing

    // First search for the target node to remove.
    if      (data < ptr->data) remove(data, ptr->left);
    else if (data > ptr->data) remove(data, ptr->right);

    // Remove a target node with two children.
    else if (   (ptr->left  != nullptr)
             && (ptr->right != nullptr))
    {
        // Replace the target node's data with the successor node's data.
        ptr->data = find_min(ptr->right)->data;

        // Search for and remove the successor node.
        remove(ptr->data, ptr->right);
    }

    // Remove a target node with one or no children.
    else
    {
        BinaryNode *oldNode = ptr;
        ptr = (ptr->left != nullptr) ? ptr->left
                                     : ptr->right;
        delete oldNode;
    }
}
